import pandas as pd
from intrinio.client import get


def companies(identifier=None, query=None):
    """
    Get companies with optional filtering using parameters.

    Args:
        identifier: Identifier for the legal entity or a security associated
            with the company: TICKER SYMBOL | FIGI | OTHER IDENTIFIER
        query: Search of company name or ticker symbol

    Returns:
        Dataset as a Pandas DataFrame
    """
    if identifier is not None:
        identifier = str.upper(identifier)

    return get('companies', identifier=identifier, query=query)


def securities(identifier=None, query=None, exch_symbol=None):
    """
    Get securities with optional filtering using parameters.

    Args:
        identifier: Identifier for the legal entity or a security associated
            with the company: TICKER SYMBOL | FIGI | OTHER IDENTIFIER
        query: Search of security name or ticker symbol
        exch_symbol: Exchange symbol

    Returns:
        Dataset as a Pandas DataFrame
    """
    if identifier is not None:
        identifier = str.upper(identifier)

    return get('securities', identifier=identifier, query=query,
               exch_symbol=exch_symbol)


def indices(identifier=None, query=None, type=None):
    """
    Get indices with optional filtering using parameters.

    Args:
        identifier: Intrinio symbol associated with the index
        query: Search of index name or symbol
        type: Type of indices: stock_market | economic | sic
    Returns:
        Dataset as a Pandas DataFrame
    """
    if identifier is not None:
        identifier = str.upper(identifier)

    return get('indices', identifier=identifier, query=query, type=type)


def prices(identifier, start_date=None, end_date=None, frequency='daily',
           sort_order='desc'):
    """
    Get historical stock market prices or indices.

    Args:
        identifier: Stock market symbol or index
        start_date: Start date of prices (default no filter)
        end_date: Last date (default today)
        frequency: Frequency of prices: daily (default) | weekly | monthly |
            quarterly | yearly
        sort_order: Order of prices: asc | desc (default)

    Returns:
        Dataset as a Pandas DataFrame
    """
    if identifier is not None:
        identifier = str.upper(identifier)

    df = get('prices', identifier=identifier, start_date=start_date,
             end_date=end_date, frequency=frequency, sort_order=sort_order)
    df.index = pd.DatetimeIndex(df.date)
    df.drop('date', axis=1, inplace=True)
    return df


def news(identifier):
    """
    Get news for a company.

    Args:
        identifier: stock market ticker symbol associated with the company's
            common stock. If the company is foreign, use the stock exchange
            code, followed by a colon, then the ticker.

    Returns:
        Dataset as a Pandas DataFrame
    """
    return get('news', identifier=str.upper(identifier))



import io
from setuptools import setup

# Version
version = open('VERSION.txt').read()

# Long description
try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
    long_description = long_description.replace("\r", "")
except (OSError, ImportError):
    print("Pandoc not found. Conversion of README.md to rst not available.")
    with io.open('README.md', encoding="utf-8") as f:
        long_description = f.read()

# Setup
setup(name='intrinio',
      version=version,
      description='Intrinio API client',
      long_description=long_description,
      keywords=['intrinio', 'API', 'data', 'client', 'quant', 'finance'],
      url='https://github.com/nhedlund/intrinio',
      author='nhedlund',
      license='MIT',
      packages=['intrinio'],
      zip_safe=False,
      install_requires=[
          'pandas >= 0.14',
          'numpy >= 1.8',
          'requests >= 2.7.0']
      )



import os
import logging
import json
import requests.sessions as sessions
import pandas as pd

username = os.getenv('INTRINIO_USERNAME')
password = os.getenv('INTRINIO_PASSWORD')

page_sizes = {
    'prices': 50000,
    'historical_data': 50000,
    'tags/standardized': 5000,
    'tags/reported': 5000,
    'tags/banks': 5000,
    'financials/standardized': 5000,
    'financials/reported': 5000,
    'financials/banks': 5000,
}

api_base_url = 'https://api.intrinio.com'
default_page_size = 250
log = logging.getLogger(__name__)


def get(endpoint, **parameters):
    """
    Get complete dataset from an endpoint using optional query parameters.

    Args:
        endpoint: Intrinio endpoint, for example: companies
        parameters: Optional query parameters

    Returns:
        Dataset as a Pandas DataFrame
    """
    page_number = 1
    dataset = pd.DataFrame()

    while True:
        page = get_page(endpoint, page_number, **parameters)
        dataset = pd.concat([dataset, page])

        if len(page) == 0 or page_number == page.total_pages:
            return dataset

        page_number += 1


def get_page(endpoint, page_number=1, page_size=None, **parameters):
    """
    Get a dataset page from an endpoint using optional query parameters.

    Args:
        endpoint: Intrinio endpoint, for example: companies
        page_number: Optional page number where 1 is the first page (default 1)
        page_size: Optional page size (default max page size for the endpoint)
        parameters: Optional query parameters

    Returns:
        Dataset page as a Pandas DataFrame with an additional total_pages
        attribute
    """
    response = query(endpoint, page_number, page_size, **parameters)

    if 'data' in response:
        page = pd.DataFrame(response['data'])
        page.total_pages = response['total_pages']
    else:
        page = pd.DataFrame([response])
        page.total_pages = 1

    return page


def query(endpoint, page_number=1, page_size=None, **parameters):
    """
    Send a query request to Intrinio API for a dataset page including page
    count and other metadata using optional query parameters.

    Args:
        endpoint: Intrinio endpoint, for example: companies
        page_number: Optional page number where 1 is the first page (default 1)
        page_size: Optional page size (default max page size for the endpoint)
        parameters: Optional query parameters

    Returns:
        Intrinio endpoint response as a tree of dictionaries, values and lists
    """
    if page_size is None:
        page_size = get_page_size(endpoint)

    url = '{}/{}'.format(api_base_url, endpoint)
    parameters['page_number'] = page_number
    parameters['page_size'] = page_size
    auth = (username, password)

    with sessions.Session() as session:
        response = session.request('GET', url, params=parameters, auth=auth,
                                   verify=True)
    if not response.ok:
        response.raise_for_status()

    return json.loads(response.content.decode('utf-8'))


def get_page_size(endpoint):
    """
    Get page size for a specific endpoint.

    Args:
        endpoint: Intrinio endpoint, for example: companies

    Returns:
        Page size as number of rows
    """
    return page_sizes.get(endpoint.lower(), default_page_size)
